#!/bin/sh
#
# KB package rollback script.
#  
# weilin95852,Sangfor
#
# 20140425: modify services of start and stop
#
#

EXCLUDE_APPLIST="dataplane networkd"
SERVICE_APPLIST=`awk -F'[][]' '/\[/{print $2}' /wns/etc/sysconfig/appinit.conf`
# 
wns_stop_services()
{
    mount -o remount,rw /
    #/wns/etc/init.d/appinitd stop
    for app in $SERVICE_APPLIST
    do
        if echo $EXCLUDE_APPLIST | grep -q -v $app; then
            pidof $app && appinit_cli stop $app >/dev/null 2>&1
        fi
    done
}

wns_start_services()
{
    #/wns/etc/init.d/appinitd restart
    #for app in $SERVICE_APPLIST
    #do
    #    if echo $EXCLUDE_APPLIST | grep -q -v $app; then
    #        if pidof $app; then
    #            appinit_cli restart $app >/dev/null 2>&1
    #        else
    #            appinit_cli start $app >/dev/null 2>&1
    #        fi
    #    fi
    #done
    #/wns/etc/init.d/appinitd restart
    /wns/sbin/appinit_cli restartall
    mount / -o ro,remount
    exit 0
}

# 当收到CTL-C中断时，重启所有服务。
trap wns_start_services INT

# 当前目录
wns_backup_home="/wns/backup"
# 回滚日志文件
wns_rollback_log_file="${wns_backup_home}/wns_rollback_log"
#
wns_appversion="/app/appversion"
wns_kbversion="/app/kbversion"
wns_hwinfo="/etc/hwinfo"

# 回滚时间
wns_rollback_update=`date +%Y%m%d-%H%M%S`


# 记录日志
wns_echo()
{
    echo $1
    echo $1 >>${wns_rollback_log_file}
}
wns_echo_ex()
{
    echo -n $1
    echo -n $1 >>${wns_rollback_log_file}
}
# 请求用户确认
# $1 提示信息，只能是一行
wns_confirm()
{
    echo -n "$1""[y/n] "
    echo -n "$1""[y/n] " >>${wns_rollback_log_file}

	while read confirm_ret
	do
		if [ "$confirm_ret" == "Y" ] || [ "$confirm_ret" == "y" ]; then
            echo "$confirm_ret" >>${wns_rollback_log_file}
			return 0
		else 
			if [ "$confirm_ret" == "N" ] || [ "$confirm_ret" == "n" ]; then 
                echo "$confirm_ret" >>${wns_rollback_log_file}
				break
			fi
		fi
        echo -n "$1""[y/n] "
	done

	return 1
}

# 软件版本检测
# 描述：
# 1 备份目录中appversion文件不存在，提示并失败返回。
# 2 大版本不一致，提示并失败返回。（大版本一定要一致）
# $1 备份目录中的appversion 绝对路径
# 成功0，失败1
wns_rollback_check_version()
{
    local bak_appversion=$1

    if [ -e $bak_appversion ]; then 
        if [ -f $bak_appversion ]; then
            local tmp_bakver=`head -n 1 $bak_appversion | awk -F- '{print $1}'`
            local bak_ver=${tmp_bakver%.*}
            local tmp_curver=`head -n 1 $wns_appversion | awk -F- '{print $1}'`
            local cur_ver=${tmp_curver%.*}
            if [ "x$bak_ver" == "x$cur_ver" ]; then
                return 0
            else
                wns_echo "backup file [$bak_appversion]'s version mismatch."
                return 1
            fi
        else
            wns_echo "backup file [$bak_appversion] not a file."
            return 1
        fi
    else
        return 1
    fi
}
# 硬件平添检测
# 描述：
# 1 备份目录中hwinfo文件不存在，提示用户是否要继续，否则返回
# 2 硬件型号不一致，提示用户是否继续，否则返回。
# $1 备份目录中的hwinfo文件，绝对路径
# 成功0，失败1
wns_rollback_check_hwinfo()
{
    local bak_hwinfo=$1

    if [ -e $bak_hwinfo ]; then #存在
        if [ -f $bak_hwinfo ]; then 
            local bak_model=`grep model= $bak_hwinfo | awk -F= '{print $2}'`
            local cur_model=`grep model= $wns_hwinfo | awk -F= '{print $2}'`
            if [ "x$bak_model" == "x$cur_model" ]; then #硬件平台不一致。正常情况应该是一致的。
                return 0
            else
                wns_echo "backup file [$bak_hwinfo]'s model not support."
                return 1
            fi
        else
            wns_echo "backup file [$bak_hwinfo] not a file."
            return 1
        fi
    else
        return 1
    fi
}

# 检测old_list,new_list文件是否一致，然后检查bak目录下文件是否一致。
# 描述：
# 1 检查old_list文件是否被修改过
# 2 检查new_list文件是否被修改过
# 3 检查old_list对应的bak下的文件是否被修改过。
# $1 备份目录
# 成功0，bak中有文件不存在或者被修改过，1
wns_rollback_check_file()
{
    local tmp_dir=$1
    local new_list="$tmp_dir/.new_list"
    local old_list="$tmp_dir/.old_list"
    local bak_dir="$tmp_dir/bak"
    local check_ret=0
    
    # 在这里可以检验bak下的文件是否被修改过。
    if [ -f $old_list ]; then
        for record in `cat $old_list`
        do
            if [ -z "$record" ]; then
                continue
            fi
            local md5_val=`echo $record | cut -d":" -f 1`
            local md5_fil=`echo $record | cut -d":" -f 2`
            local bakfile="$bak_dir/$md5_fil"

            wns_echo_ex "Check $bakfile ...... "
            # 文件不存在
            if [ -f "$bakfile" ]; then
                # 检测下备份文件是否被修改过
                local file_md5_result=`md5sum $bakfile`
                local file_md5=`echo "${file_md5_result}" | cut -d" " -f 1`
                if [ "$file_md5" != "$md5_val" ]; then
                    wns_echo "modified"
                    check_ret=1
                    continue
                fi
            else
                wns_echo "not exist"
                check_ret=1
                continue
            fi
            wns_echo "OK"
        done
    fi
    
    if [ "$check_ret" == "1" ]; then
    	return 1
    fi

    return 0
}

# 根据new_list文件删除设备中文件：打KB包增加的文件和替换后的文件
# $1 设备中文件
wns_rollback_delete_file()
{
    if [ -f $1 ]; then
        #rm -f $1
        unlink $1
        wns_echo "delete file $1"
    fi
}
# 根据old_list文件覆盖文件
# $1 bak中文件
wns_rollback_cover_file()
{
    local dev_file=$2;
    local bak_file="${1}/${2}"

    #rm -f ${dev_file}>/dev/null
    unlink ${dev_file}>/dev/null

    if [ -f ${bak_file} ]; then
        cp -f ${bak_file} ${dev_file}
        wns_echo "cover file ${bak_file} --> ${dev_file}"
    fi
}

# 回滚指定目录完成后删除该目录
# $1 指定目录
wns_rollback_delete_dir()
{
    if [ -d $1 ]; then
        rm -rf $1
        wns_echo "delete directory $1 "
    fi
}

# 检查目录是否是备份目录
# 要求有.new_list文件 .old_list文件 bak目录
wns_rollback_dir_ok()
{
    local tmp_dir=$1
    local new_list="$tmp_dir/.new_list"
    local old_list="$tmp_dir/.old_list"
    local bak_dir="$tmp_dir/bak"

    # 检测这三个文件是否都存在，都存在则认为是备份目录
    if [ -f $new_list ] && [ -f $old_list ] && [ -d $bak_dir ]; then
        return 0 
    else
        wns_echo "$tmp_dir is not backup directory"
        return 1
    fi
}

# 回滚指定的目录
# 描述：
# 1 检查指定目录是否存在，是否是目录
# 2 检查软件版本
# 3 检查硬件平台
# 4 检查old，new文件是否一致
# 5 开始回滚
# 6 返回结果
# $1指定目录
wns_rollback_do_dir()
{
    local tmp_dir=$1

    wns_echo ""
    wns_echo "Start to check $tmp_dir"
    # 检测目录是否存在
    if [ ! -d $tmp_dir ]; then
        wns_echo "$tmp_dir is not directory."
        return 1
    fi
    # 检测目录是否是备份目录
    wns_rollback_dir_ok $tmp_dir
    if [ $? -ne 0 ]; then
        return 1
    fi
    # 检测版本
    wns_rollback_check_version "$tmp_dir/bak/app/appversion" 
    if [ $? -ne 0 ]; then
        return 1
    fi
    # 检测硬件平台
    wns_rollback_check_hwinfo "$tmp_dir/bak/etc/hwinfo" 
    if [ $? -ne 0 ]; then
        return 1
    fi
    # 检测文件
    wns_rollback_check_file "$tmp_dir" 
    if [ $? -ne 0 ]; then
        return 1
    fi

    wns_echo "Start to rollback"

    #开始执行回滚脚本
    if [ -f "$tmp_dir/KB-WACAP-rollback-custom.sh" ]; then
	sh $tmp_dir/KB-WACAP-rollback-custom.sh
    fi

    # 开始删除文件
    local new_list="$tmp_dir/.new_list"
    if [ -f $new_list ]; then
        for item_del in `cat $new_list`
        do
            local tmp_file=`echo $item_del | cut -d ":" -f 2`
            wns_rollback_delete_file $tmp_file
        done
    fi
    # 开始恢复文件
    local old_list="$tmp_dir/.old_list"
    if [ -f $old_list ]; then
        for item_add in `cat $old_list`
        do
            local tmp_file=`echo $item_add | cut -d ":" -f 2`
            wns_rollback_cover_file "$tmp_dir/bak" "$tmp_file"
        done
    fi

    # 回滚完毕，删除备份目录
    wns_rollback_delete_dir $tmp_dir

    wns_echo "Rollback $tmp_dir success."
}

# 获取备份目录信息
wns_rollback_dir_info()
{
    local tmp_file="$1/.info_list"
    
    if [ -f "$tmp_file" ]; then
        local author=`grep author= $tmp_file | awk -F= '{print $2}'`
        local update=`grep datetime= $tmp_file | awk -F= '{print $2}'`
        local kbflag=`grep kbflag= $tmp_file | awk -F= '{print $2}'`
        local dept=`grep dept= $tmp_file | awk -F= '{print $2}'`
        local tdnum=`grep TDnumber= $tmp_file | awk -F= '{print $2}'`
    
        if [ "$tdnum" == "" ]; then
            tdnum="null"
        fi

        echo "`basename $1`   $kbflag            $update     $author($dept)  $tdnum"
    fi
}

# 罗列备份目录信息
wns_rollback_dir_list()
{
    echo "dir   KB flag    update time  author(depart)    TDnumber"
    echo "----  --------------------------------------------------------"
    for item_info in `ls -l $wns_backup_home | grep ^d | awk '{print $9}' | sort -r`
    do
        wns_rollback_dir_info "$wns_backup_home/$item_info"
    done
}

# 回滚到指定目录
# $1是目录名称
wns_rollback_to_me()
{
    # 回滚到指定目录
    local to_me=`basename $1`
    # 当前的目录列表
    local dirlist=`ls -l $wns_backup_home | grep ^d | awk '{print $9}' | sort -r`
    local found=0

    # 检测回滚到的目录是否存在
    for item_found in $dirlist
    do
        if [ "$to_me" == "$item_found" ]; then
            found=1
        fi
    done
    if [ $found -eq 0 ]; then
        echo "Sorry! $to_me not exist. exit."
        exit 0
    fi

    # 开始回滚
    for item_dir in  $dirlist
    do
        wns_rollback_do_dir "$wns_backup_home/$item_dir"
		#modify by cz 2015-1-26 for TD 181
		#原来的脚本没有对wns_rollback_do_dir的返回值作判断，这显然会出问题的 --add by cz 20150123
		if [ $? -ne 0 ] ;then
			wns_echo "Rollback to dir:[$wns_backup_home/$item_dir] fail !  exit now..."
			sleep 2
			exit 0
		fi
		
        # 需要处理下一个目录吗
        #echo "--------------------------------------"
        if [ "$to_me" == "$item_dir" ]; then
            break
        fi
        #echo "======================================"
    done
}



wns_rollback_help()
{
    echo "Usage: xxx.sh [option]"
    echo "   help : display this help and exit."
    echo "   list : display informations of all backup directory."
    echo "   <dir>: directory name. rollback to the directory(include it)"
    echo ""
}





#####################################################################
# 开始
###############################################

if [ $# -ne 1 ]; then
    wns_rollback_help
    exit 0
fi

param=$1

if [ "$param" == "help" ]; then
    wns_rollback_help
    exit 0
else 
    if [ "$param" == "list" ]; then
        wns_rollback_dir_list
        exit 0
    else
        wns_stop_services
        wns_echo "-------------------------------------------"
        wns_echo "start time: $wns_rollback_update"
        wns_rollback_to_me $param
        sync
    fi
fi

# 删除回滚提示语文件
alertFile="/wns/backup/KB-WACAP-rollback-alert.txt"
if [ -f "$alertFile" ]; then 
    rm "$alertFile" 
fi

# 执行回滚脚本
wns_function_before_rollback
#mount -o remount rw /
#sinforauthtool -op 2 /wns/webui/application/views/Welcome/login.php

wns_echo "rollback successfully"

# 重启确认
if wns_confirm "Please confirm for reboot now:"; then
    for((wns_idx=0; wns_idx<10; ++wns_idx))
    do
        wns_left_time=`expr 10 - $wns_idx`
        wns_echo "WARN!!! Reboot after ${wns_left_time}s (Ctrl+c for cancel reboot)"
        sleep 1
    done
    wns_echo "reboot now"
    reboot
fi

wns_start_services

exit 0

